#undef WIN32
#undef WIN32_DLOPEN

#undef ENABLE_MD5
#undef ENABLE_SHA1
#undef ENABLE_TIGER
#undef ENABLE_HAVAL
#undef ENABLE_CRC32
#undef ENABLE_RIPEMD
#undef ENABLE_MD4
#undef ENABLE_GOST
#undef ENABLE_ADLER32
#undef ENABLE_SHA256_SHA224
#undef ENABLE_SHA512_SHA384
#undef ENABLE_WHIRLPOOL
#undef ENABLE_SNEFRU
#undef ENABLE_MD2
